import { echo } from "./commands/echo";
import { ls } from "./commands/ls";
import { pwd } from "./commands/pwd";
import { cd } from "./commands/cd";
import { touch } from "./commands/touch";
import { cat } from "./commands/cat";
import { rm } from "./commands/rm";
import { IShellCommand } from "./commands/IShellCommand";
import { FileSystem } from "./FileSystem";
import { ShellSyscall } from "./ShellSyscall";

// System class to manage commands
class System {
    private static instance: System | null = null;
    private commands: Array<IShellCommand>;
    private fs: FileSystem;
    private cwd: string;

    constructor() {
        this.commands = new Array<IShellCommand>();
        this.fs = new FileSystem();
        this.cwd = '/';
        this.initializeCommands();
    }

    private initializeCommands(): void {
        const syscall = new ShellSyscall(this.fs, this.cwd);
        this.commands.push(new echo());
        this.commands.push(new ls(syscall));
        this.commands.push(new pwd(syscall));
        this.commands.push(new cd(syscall));
        this.commands.push(new touch(syscall));
        this.commands.push(new cat(syscall));
        this.commands.push(new rm(syscall));
    }

    public static getInstance(): System {
        if (!System.instance) {
            System.instance = new System();
        }
        return System.instance;
    }

    knowsCommand(commandName: string): boolean {
        return this.commands.some(c => c.getName() === commandName);
    }

    executeCommand(commandName: string, args: string[]): string {
        const command = this.commands.find(c => c.getName() === commandName);
        if (command) {
            return command.execute(args);
        }
        return `Command not found: ${commandName}`;
    }
}

// Export the System class and getInstance for use in other files
export { System };